"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
/**
 * Pass events to a given observable subject without triggering change detection
 *
 * Usage:
 * <button [ev-observe]="subject" ev-events="click"></button>
 *
 * ...
 *
 * export class ExampleComponent implements OnInit {
 *  public subject = new Subject();
 *
 *  constructor(private zone: NgZone) {}
 *
 *  ngOnInit() {
 *      this.subject
 *          .throttleTime(300)
 *          .filter(someCondition)
 *          ...
 *          .subscribe($event => this.zone.run(() => this.handleEvent($event)));
 *  }
 * }
 *
 * Notes:
 * The [ev-events] property can be either a string or an array of strings to handle multiple events
 *
 */
var ObserveEventDirective = (function () {
    /**
     * @param {?} elm
     * @param {?} renderer
     * @param {?} zone
     */
    function ObserveEventDirective(elm, renderer, zone) {
        this.elm = elm;
        this.renderer = renderer;
        this.zone = zone;
        this.listeners = [];
    }
    Object.defineProperty(ObserveEventDirective.prototype, "events", {
        /**
         * @param {?} e
         * @return {?}
         */
        set: function (e) {
            var _this = this;
            this.unregisterAll();
            var /** @type {?} */ events = (typeof (e) === 'string')
                ? [e]
                : e;
            this.zone.runOutsideAngular(function () {
                _this.listeners = events.map(function (x) { return _this.addListener(x); });
            });
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    ObserveEventDirective.prototype.ngOnDestroy = function () {
        this.unregisterAll();
    };
    /**
     * @return {?}
     */
    ObserveEventDirective.prototype.unregisterAll = function () {
        this.listeners.forEach(function (x) { return x(); });
        this.listeners = [];
    };
    /**
     * @param {?} eventName
     * @return {?}
     */
    ObserveEventDirective.prototype.addListener = function (eventName) {
        var _this = this;
        var /** @type {?} */ colon = eventName.indexOf(':');
        var /** @type {?} */ handler = function ($event) { return _this.fireEvent($event); };
        if (colon === -1) {
            return this.renderer.listen(this.elm.nativeElement, eventName, handler);
        }
        else {
            var /** @type {?} */ scope = eventName.slice(0, colon);
            var /** @type {?} */ realEventName = eventName.slice(colon + 1);
            return this.renderer.listenGlobal(scope, realEventName, handler);
        }
    };
    /**
     * @param {?} $event
     * @return {?}
     */
    ObserveEventDirective.prototype.fireEvent = function ($event) {
        if (!this.observe)
            return;
        this.observe.next($event);
    };
    return ObserveEventDirective;
}());
ObserveEventDirective.decorators = [
    { type: core_1.Directive, args: [{
                selector: '[ev-observe]'
            },] },
];
/**
 * @nocollapse
 */
ObserveEventDirective.ctorParameters = function () { return [
    { type: core_1.ElementRef, },
    { type: core_1.Renderer, },
    { type: core_1.NgZone, },
]; };
ObserveEventDirective.propDecorators = {
    'observe': [{ type: core_1.Input, args: ['ev-observe',] },],
    'events': [{ type: core_1.Input, args: ['ev-events',] },],
};
exports.ObserveEventDirective = ObserveEventDirective;
function ObserveEventDirective_tsickle_Closure_declarations() {
    /** @type {?} */
    ObserveEventDirective.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    ObserveEventDirective.ctorParameters;
    /** @type {?} */
    ObserveEventDirective.propDecorators;
    /** @type {?} */
    ObserveEventDirective.prototype.observe;
    /** @type {?} */
    ObserveEventDirective.prototype.listeners;
    /** @type {?} */
    ObserveEventDirective.prototype.elm;
    /** @type {?} */
    ObserveEventDirective.prototype.renderer;
    /** @type {?} */
    ObserveEventDirective.prototype.zone;
}
